TOOL_PARAM_MAP = {
    # --- Authentication ---
    "mcp_cldkctl_cldkctl_auth": {
        "endpoint": "/core/cldkctl/auth",
        "method": "POST",
        "required_params": [
            {"name": "token", "type": "string", "desc": "Your cldkctl token (starts with 'cldkctl_')"},
        ],
        "optional_params": [
            {"name": "force_staging", "type": "boolean", "desc": "Force using staging environment (default: false)"},
        ],
    },
    # --- Auth & Profile ---
    "mcp_cldkctl_cldkctl_login": {
        "endpoint": "/core/user/login",
        "method": "POST",
        "required_params": [
            {"name": "username", "type": "string", "desc": "User's login name"},
            {"name": "password", "type": "string", "desc": "User's password"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_profile_detail": {
        "endpoint": "/core/user/profile",
        "method": "GET",
        "required_params": [],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_update_profile": {
        "endpoint": "/core/user/organization/profile/member/:user_id",
        "method": "PUT",
        "required_params": [
            {"name": "user_id", "type": "string", "desc": "ID of the user to update"},
            {"name": "name", "type": "string", "desc": "Full name"},
            {"name": "email", "type": "string", "desc": "Email address"},
            {"name": "phone_number", "type": "string", "desc": "Phone number"},
            {"name": "address", "type": "string", "desc": "Address"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_change_password": {
        "endpoint": "/core/user/change-password",
        "method": "POST",
        "required_params": [
            {"name": "old_password", "type": "string", "desc": "Current password"},
            {"name": "new_password", "type": "string", "desc": "New password"},
        ],
        "optional_params": [],
    },
    # --- Project Management ---
    "mcp_cldkctl_cldkctl_project_list": {
        "endpoint": "/core/user/organization/projects/byOrg",
        "method": "GET",
        "required_params": [],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_project_detail": {
        "endpoint": "/core/user/project/detail/:project_id",
        "method": "GET",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "ID of the project"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_update_project": {
        "endpoint": "/core/user/projects/:project_id",
        "method": "PUT",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "ID of the project"},
            {"name": "description", "type": "string", "desc": "New project description"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_check_before_delete_project": {
        "endpoint": "/core/user/checking/projects/:project_id",
        "method": "DELETE",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "ID of the project"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_delete_project": {
        "endpoint": "/core/user/projects/:project_id",
        "method": "DELETE",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "ID of the project"},
        ],
        "optional_params": [],
    },
    # --- Balance & Billing ---
    "mcp_cldkctl_cldkctl_balance_detail": {
        "endpoint": "/core/balance/accumulated/:project_id",
        "method": "GET",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "ID of the project"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_payment_history": {
        "endpoint": "/core/payment/history",
        "method": "GET",
        "required_params": [],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_billing_daily_cost": {
        "endpoint": "/core/billing/v2/daily-cost/:project_id",
        "method": "GET",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "ID of the project"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_billing_monthly_cost": {
        "endpoint": "/core/billing/monthly-cost/total-billed/:project_id",
        "method": "GET",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "ID of the project"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_billing_history": {
        "endpoint": "/core/billing/monthly-cost/history",
        "method": "POST",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "ID of the project"},
            {"name": "start", "type": "string", "desc": "Start date (YYYY-MM-DD)"},
            {"name": "end", "type": "string", "desc": "End date (YYYY-MM-DD)"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_billing_invoice_sme": {
        "endpoint": "/core/balance/history/invoice",
        "method": "GET",
        "required_params": [],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_billing_invoice_sme_detail": {
        "endpoint": "/core/balance/history/invoice/detail/:invoice_id",
        "method": "GET",
        "required_params": [
            {"name": "invoice_id", "type": "string", "desc": "ID of the invoice"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_billing_invoice_enterprise": {
        "endpoint": "/core/billing/invoice/:project_id",
        "method": "GET",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "ID of the project"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_billing_invoice_enterprise_detail": {
        "endpoint": "/core/billing/v2/invoice/detail/:invoice_id",
        "method": "GET",
        "required_params": [
            {"name": "invoice_id", "type": "string", "desc": "ID of the invoice"},
        ],
        "optional_params": [],
    },
    # --- Organization & Members ---
    "mcp_cldkctl_cldkctl_org_detail": {
        "endpoint": "/core/user/organization",
        "method": "GET",
        "required_params": [],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_org_members": {
        "endpoint": "/core/user/organization/member",
        "method": "GET",
        "required_params": [],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_org_member_add": {
        "endpoint": "/core/user/organization/member",
        "method": "POST",
        "required_params": [
            {"name": "user_id", "type": "string", "desc": "ID of the user to add"},
            {"name": "role_id", "type": "string", "desc": "Role ID to assign"},
            {"name": "project_id", "type": "string", "desc": "Project ID to assign"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_org_member_edit": {
        "endpoint": "/core/user/organization/member/:user_id",
        "method": "PUT",
        "required_params": [
            {"name": "user_id", "type": "string", "desc": "ID of the user to edit"},
            {"name": "role_id", "type": "string", "desc": "Role ID to assign"},
            {"name": "project_id", "type": "string", "desc": "Project ID to assign"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_org_member_delete": {
        "endpoint": "/core/user/organization/member/:user_id",
        "method": "DELETE",
        "required_params": [
            {"name": "user_id", "type": "string", "desc": "ID of the user to delete"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_org_role_list": {
        "endpoint": "/core/user/organization/role",
        "method": "GET",
        "required_params": [],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_org_role_add": {
        "endpoint": "/core/user/organization/role",
        "method": "POST",
        "required_params": [
            {"name": "name", "type": "string", "desc": "Role name"},
            {"name": "privileges", "type": "list", "desc": "List of privileges"},
        ],
        "optional_params": [],
    },
    # --- Kubernetes ---
    "mcp_cldkctl_cldkctl_k8s_pods": {
        "endpoint": "/core/pods",
        "method": "GET",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_k8s_deployments": {
        "endpoint": "/core/deployment",
        "method": "GET",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_k8s_services": {
        "endpoint": "/core/kubernetes/services",
        "method": "GET",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_k8s_configmaps": {
        "endpoint": "/core/kubernetes/configmap",
        "method": "GET",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_k8s_secrets": {
        "endpoint": "/core/kubernetes/secret",
        "method": "GET",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
        ],
        "optional_params": [],
    },
    # --- VM Management ---
    "mcp_cldkctl_cldkctl_vm_list": {
        "endpoint": "/core/virtual-machine/list/all",
        "method": "GET",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "limit", "type": "string", "desc": "Limit (use -1 for all)"}
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_vm_detail": {
        "endpoint": "/core/virtual-machine/detail-vm",
        "method": "POST",
        "required_params": [
            {"name": "vm_id", "type": "string", "desc": "ID of the VM"},
        ],
        "optional_params": [],
    },
    # --- Registry ---
    "mcp_cldkctl_cldkctl_registry_list": {
        "endpoint": "/core/dekaregistry/v2/registry",
        "method": "GET",
        "required_params": [],
        "optional_params": [
            {"name": "project-id", "type": "string", "desc": "Project ID (hyphenated, as required by backend)"},
            {"name": "page", "type": "string", "desc": "Page number (default 1)"}
        ],
    },
    "mcp_cldkctl_cldkctl_registry_repositories": {
        "endpoint": "/core/dekaregistry/v2/repository",
        "method": "GET",
        "required_params": [
            {"name": "registry_id", "type": "string", "desc": "Registry ID"},
        ],
        "optional_params": [],
    },
    # --- Notebook ---
    "mcp_cldkctl_cldkctl_notebook_list": {
        "endpoint": "/core/deka-notebook",
        "method": "GET",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_notebook_create": {
        "endpoint": "/core/deka-notebook",
        "method": "POST",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "name", "type": "string", "desc": "Notebook name"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
            {"name": "image", "type": "string", "desc": "Image ID"},
            {"name": "cpu", "type": "string", "desc": "CPU amount"},
            {"name": "memory", "type": "string", "desc": "Memory size"},
            {"name": "storage", "type": "string", "desc": "Storage size"},
        ],
        "optional_params": [
            {"name": "gpu", "type": "string", "desc": "GPU amount (optional)"},
        ],
    },
    # --- More Kubernetes ---
    "mcp_cldkctl_cldkctl_k8s_pod_create": {
        "endpoint": "/core/pods",
        "method": "POST",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
            {"name": "spec", "type": "dict", "desc": "Pod spec (YAML/JSON)"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_k8s_pod_edit": {
        "endpoint": "/core/pods/:project_id/:namespace/:name",
        "method": "PUT",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
            {"name": "name", "type": "string", "desc": "Pod name"},
            {"name": "spec", "type": "dict", "desc": "Pod spec (YAML/JSON)"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_k8s_pod_delete": {
        "endpoint": "/core/pods/:project_id/:namespace/:name",
        "method": "DELETE",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
            {"name": "name", "type": "string", "desc": "Pod name"},
        ],
        "optional_params": [],
    },
    # --- More VM Management ---
    "mcp_cldkctl_cldkctl_vm_create": {
        "endpoint": "/core/virtual-machine",
        "method": "POST",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "name", "type": "string", "desc": "VM name"},
            {"name": "flavor", "type": "string", "desc": "Flavor ID"},
            {"name": "image", "type": "string", "desc": "Image ID"},
            {"name": "network", "type": "string", "desc": "Network ID"},
            {"name": "storage", "type": "string", "desc": "Storage size"},
        ],
        "optional_params": [
            {"name": "gpu", "type": "string", "desc": "GPU type (optional)"},
        ],
    },
    "mcp_cldkctl_cldkctl_vm_delete": {
        "endpoint": "/core/virtual-machine/delete",
        "method": "POST",
        "required_params": [
            {"name": "vm_id", "type": "string", "desc": "ID of the VM to delete"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_vm_reboot": {
        "endpoint": "/core/virtual-machine/reboot",
        "method": "POST",
        "required_params": [
            {"name": "vm_id", "type": "string", "desc": "ID of the VM to reboot"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_vm_turn_on": {
        "endpoint": "/core/virtual-machine/turn-on/vm",
        "method": "POST",
        "required_params": [
            {"name": "vm_id", "type": "string", "desc": "ID of the VM to turn on"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_vm_turn_off": {
        "endpoint": "/core/virtual-machine/turn-off/vm",
        "method": "POST",
        "required_params": [
            {"name": "vm_id", "type": "string", "desc": "ID of the VM to turn off"},
        ],
        "optional_params": [],
    },
    # --- More Registry ---
    "mcp_cldkctl_cldkctl_registry_detail": {
        "endpoint": "/core/dekaregistry/v2/registry/:registry_id",
        "method": "GET",
        "required_params": [
            {"name": "registry_id", "type": "string", "desc": "Registry ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_overview": {
        "endpoint": "/core/dekaregistry/v2/registry/:registry_id/overview",
        "method": "GET",
        "required_params": [
            {"name": "registry_id", "type": "string", "desc": "Registry ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_member_list": {
        "endpoint": "/core/dekaregistry/v2/member/:registry_id",
        "method": "GET",
        "required_params": [
            {"name": "registry_id", "type": "string", "desc": "Registry ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_member_add": {
        "endpoint": "/core/dekaregistry/v2/member/:registry_id",
        "method": "POST",
        "required_params": [
            {"name": "registry_id", "type": "string", "desc": "Registry ID"},
            {"name": "user_id", "type": "string", "desc": "User ID to add"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_member_delete": {
        "endpoint": "/core/dekaregistry/v2/member/:registry_id/detail/:member_id",
        "method": "DELETE",
        "required_params": [
            {"name": "registry_id", "type": "string", "desc": "Registry ID"},
            {"name": "member_id", "type": "string", "desc": "Member ID to delete"},
        ],
        "optional_params": [],
    },
    # --- More Notebook ---
    "mcp_cldkctl_cldkctl_notebook_delete": {
        "endpoint": "/core/deka-notebook/delete",
        "method": "POST",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "name", "type": "string", "desc": "Notebook name"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_notebook_update": {
        "endpoint": "/core/deka-notebook/yaml",
        "method": "PUT",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "name", "type": "string", "desc": "Notebook name"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
            {"name": "yaml", "type": "string", "desc": "Notebook YAML"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_notebook_start": {
        "endpoint": "/core/deka-notebook/start",
        "method": "POST",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "name", "type": "string", "desc": "Notebook name"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_notebook_stop": {
        "endpoint": "/core/deka-notebook/stop",
        "method": "POST",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "name", "type": "string", "desc": "Notebook name"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
        ],
        "optional_params": [],
    },
    # --- More Billing/Quota ---
    "mcp_cldkctl_cldkctl_project_r_quota_post": {
        "endpoint": "/mid/billing/projectdekagpu/quota/:project_id",
        "method": "GET",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_project_r_quota_pre": {
        "endpoint": "/mid/billing/projectflavorgpu/project/:project_id",
        "method": "GET",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
        ],
        "optional_params": [],
    },
    # --- More Token Management ---
    "mcp_cldkctl_cldkctl_token_update": {
        "endpoint": "/core/cldkctl/token/:token_id",
        "method": "PUT",
        "required_params": [
            {"name": "token_id", "type": "string", "desc": "Token ID to update"},
            {"name": "name", "type": "string", "desc": "New token name"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_token_regenerate": {
        "endpoint": "/core/cldkctl/token/regenerate/:token_id",
        "method": "POST",
        "required_params": [
            {"name": "token_id", "type": "string", "desc": "Token ID to regenerate"},
            {"name": "expiration_days", "type": "integer", "desc": "Expiration in days"},
        ],
        "optional_params": [],
    },
    # --- Miscellaneous ---
    "mcp_cldkctl_cldkctl_kube_dashboard": {
        "endpoint": "/core/user/projects/:project_id/vcluster/dashboard",
        "method": "GET",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_kubeconfig": {
        "endpoint": "/core/user/projects/:project_id/vcluster/kubeconfig",
        "method": "GET",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_get_namespace": {
        "endpoint": "/core/user/projects/:project_id/vcluster/namespaces",
        "method": "GET",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
        ],
        "optional_params": [],
    },
    # --- Missing tools from endpoint map ---
    "mcp_cldkctl_cldkctl_org_edit": {
        "endpoint": "/core/user/organization/edit/:organization_id",
        "method": "PUT",
        "required_params": [
            {"name": "organization_id", "type": "string", "desc": "Organization ID"},
            {"name": "org_data", "type": "object", "desc": "Organization data to update"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_org_member_activate": {
        "endpoint": "/core/user/manageuser/active/:user_id",
        "method": "PUT",
        "required_params": [
            {"name": "user_id", "type": "string", "desc": "User ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_org_member_deactivate": {
        "endpoint": "/core/user/manageuser/deactive/:user_id",
        "method": "PUT",
        "required_params": [
            {"name": "user_id", "type": "string", "desc": "User ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_org_member_resend_invitation": {
        "endpoint": "/core/superadmin/manageuser/resend-verified/:user_id",
        "method": "POST",
        "required_params": [
            {"name": "user_id", "type": "string", "desc": "User ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_voucher_claim": {
        "endpoint": "/core/user/voucher-credit/claim",
        "method": "POST",
        "required_params": [
            {"name": "claim_data", "type": "object", "desc": "Claim data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_voucher_claimed_list": {
        "endpoint": "/core/user/voucher-credit/claimed",
        "method": "GET",
        "required_params": [],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_voucher_trial_claimed_list": {
        "endpoint": "/core/user/voucher/claimed",
        "method": "GET",
        "required_params": [],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_audit_log": {
        "endpoint": "/core/api/v1.1/user/activity/sp/get-auditlog",
        "method": "GET",
        "required_params": [],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_org_role_detail": {
        "endpoint": "/core/user/organization/role/:role_id",
        "method": "GET",
        "required_params": [
            {"name": "role_id", "type": "string", "desc": "Role ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_org_role_edit": {
        "endpoint": "/core/user/organization/role/:role_id",
        "method": "PUT",
        "required_params": [
            {"name": "role_id", "type": "string", "desc": "Role ID"},
            {"name": "role_data", "type": "object", "desc": "Role data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_org_role_delete": {
        "endpoint": "/core/user/organization/role/:role_id",
        "method": "DELETE",
        "required_params": [
            {"name": "role_id", "type": "string", "desc": "Role ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_org_role_add": {
        "endpoint": "/core/user/organization/role",
        "method": "POST",
        "required_params": [
            {"name": "role_data", "type": "object", "desc": "Role data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_kube_dashboard": {
        "endpoint": "/core/user/projects/:project_id/vcluster/dashboard",
        "method": "GET",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_kubeconfig": {
        "endpoint": "/core/user/projects/:project_id/vcluster/kubeconfig",
        "method": "GET",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
        ],
        "optional_params": [],
    },
    # --- More missing tools from endpoint map ---
    "mcp_cldkctl_cldkctl_get_pod": {
        "endpoint": "/core/pods",
        "method": "GET",
        "required_params": [],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_create_pod": {
        "endpoint": "/core/pods",
        "method": "POST",
        "required_params": [
            {"name": "pod_data", "type": "object", "desc": "Pod data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_edit_pod": {
        "endpoint": "/core/pods/:project_id/:namespace/:name",
        "method": "PUT",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
            {"name": "name", "type": "string", "desc": "Pod name"},
            {"name": "pod_data", "type": "object", "desc": "Pod data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_delete_pod": {
        "endpoint": "/core/pods/:project_id/:namespace/:name",
        "method": "DELETE",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
            {"name": "name", "type": "string", "desc": "Pod name"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_console_pod": {
        "endpoint": "/core/pods/console/:token",
        "method": "GET",
        "required_params": [
            {"name": "token", "type": "string", "desc": "Console token"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_console_token_pod": {
        "endpoint": "/core/pods/console",
        "method": "POST",
        "required_params": [
            {"name": "console_data", "type": "object", "desc": "Console data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_get_deployment": {
        "endpoint": "/core/deployment",
        "method": "GET",
        "required_params": [],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_create_deployment": {
        "endpoint": "/core/deployment",
        "method": "POST",
        "required_params": [
            {"name": "deployment_data", "type": "object", "desc": "Deployment data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_edit_deployment": {
        "endpoint": "/core/deployment/:project_id/:namespace/:name",
        "method": "PUT",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
            {"name": "name", "type": "string", "desc": "Deployment name"},
            {"name": "deployment_data", "type": "object", "desc": "Deployment data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_delete_deployment": {
        "endpoint": "/core/deployment/:project_id/:namespace/:name",
        "method": "DELETE",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
            {"name": "name", "type": "string", "desc": "Deployment name"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_get_daemonset": {
        "endpoint": "/core/daemonset",
        "method": "GET",
        "required_params": [],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_create_daemonset": {
        "endpoint": "/core/daemonset",
        "method": "POST",
        "required_params": [
            {"name": "daemonset_data", "type": "object", "desc": "DaemonSet data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_edit_daemonset": {
        "endpoint": "/core/daemonset/:project_id/:namespace/:name",
        "method": "PUT",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
            {"name": "name", "type": "string", "desc": "DaemonSet name"},
            {"name": "daemonset_data", "type": "object", "desc": "DaemonSet data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_delete_daemonset": {
        "endpoint": "/core/daemonset/:project_id/:namespace/:name",
        "method": "DELETE",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
            {"name": "name", "type": "string", "desc": "DaemonSet name"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_get_statefulset": {
        "endpoint": "/core/statefulsets",
        "method": "GET",
        "required_params": [],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_create_statefulset": {
        "endpoint": "/core/statefulsets",
        "method": "POST",
        "required_params": [
            {"name": "statefulset_data", "type": "object", "desc": "StatefulSet data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_edit_statefulset": {
        "endpoint": "/core/statefulsets/:project_id/:namespace/:name",
        "method": "PUT",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
            {"name": "name", "type": "string", "desc": "StatefulSet name"},
            {"name": "statefulset_data", "type": "object", "desc": "StatefulSet data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_delete_statefulset": {
        "endpoint": "/core/statefulsets/:project_id/:namespace/:name",
        "method": "DELETE",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
            {"name": "name", "type": "string", "desc": "StatefulSet name"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_get_service": {
        "endpoint": "/core/kubernetes/services",
        "method": "GET",
        "required_params": [],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_create_service": {
        "endpoint": "/core/kubernetes/services",
        "method": "POST",
        "required_params": [
            {"name": "service_data", "type": "object", "desc": "Service data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_edit_service": {
        "endpoint": "/core/kubernetes/services/:project_id/:namespace/:name",
        "method": "PUT",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
            {"name": "name", "type": "string", "desc": "Service name"},
            {"name": "service_data", "type": "object", "desc": "Service data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_delete_service": {
        "endpoint": "/core/kubernetes/services/:project_id/:namespace/:name",
        "method": "DELETE",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
            {"name": "name", "type": "string", "desc": "Service name"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_get_persistent_volume": {
        "endpoint": "/core/kubernetes/pv",
        "method": "GET",
        "required_params": [],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_create_persistent_volume": {
        "endpoint": "/core/kubernetes/pv",
        "method": "POST",
        "required_params": [
            {"name": "pv_data", "type": "object", "desc": "Persistent Volume data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_edit_persistent_volume": {
        "endpoint": "/core/kubernetes/pv/:project_id/:name",
        "method": "PUT",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "name", "type": "string", "desc": "Persistent Volume name"},
            {"name": "pv_data", "type": "object", "desc": "Persistent Volume data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_delete_persistent_volume": {
        "endpoint": "/core/kubernetes/pv/:project_id/:name",
        "method": "DELETE",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "name", "type": "string", "desc": "Persistent Volume name"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_get_pvc": {
        "endpoint": "/core/kubernetes/pvc",
        "method": "GET",
        "required_params": [],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_create_pvc": {
        "endpoint": "/core/kubernetes/pvc",
        "method": "POST",
        "required_params": [
            {"name": "pvc_data", "type": "object", "desc": "Persistent Volume Claim data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_edit_pvc": {
        "endpoint": "/core/kubernetes/pvc/:project_id/:namespace/:name",
        "method": "PUT",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
            {"name": "name", "type": "string", "desc": "PVC name"},
            {"name": "pvc_data", "type": "object", "desc": "Persistent Volume Claim data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_delete_pvc": {
        "endpoint": "/core/kubernetes/pvc/:project_id/:namespace/:name",
        "method": "DELETE",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
            {"name": "name", "type": "string", "desc": "PVC name"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_get_data_volume": {
        "endpoint": "/core/datavolume",
        "method": "GET",
        "required_params": [],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_create_data_volume": {
        "endpoint": "/core/datavolume",
        "method": "POST",
        "required_params": [
            {"name": "datavolume_data", "type": "object", "desc": "Data Volume data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_edit_data_volume": {
        "endpoint": "/core/datavolume/:project_id/:namespace/:name",
        "method": "PUT",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
            {"name": "name", "type": "string", "desc": "Data Volume name"},
            {"name": "datavolume_data", "type": "object", "desc": "Data Volume data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_delete_data_volume": {
        "endpoint": "/core/datavolume/:project_id/:namespace/:name",
        "method": "DELETE",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
            {"name": "name", "type": "string", "desc": "Data Volume name"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_get_resource_v1": {
        "endpoint": "/core/kubernetes/:resource",
        "method": "GET",
        "required_params": [
            {"name": "resource", "type": "string", "desc": "Kubernetes resource type"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_create_resource_v1": {
        "endpoint": "/core/kubernetes/:resource",
        "method": "POST",
        "required_params": [
            {"name": "resource", "type": "string", "desc": "Kubernetes resource type"},
            {"name": "resource_data", "type": "object", "desc": "Resource data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_edit_resource_v1": {
        "endpoint": "/core/kubernetes/:resource/:project_id/:namespace/:name",
        "method": "PATCH",
        "required_params": [
            {"name": "resource", "type": "string", "desc": "Kubernetes resource type"},
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
            {"name": "name", "type": "string", "desc": "Resource name"},
            {"name": "resource_data", "type": "object", "desc": "Resource data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_delete_resource_v1": {
        "endpoint": "/core/kubernetes/:resource/:project_id/:namespace/:name",
        "method": "DELETE",
        "required_params": [
            {"name": "resource", "type": "string", "desc": "Kubernetes resource type"},
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
            {"name": "name", "type": "string", "desc": "Resource name"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_get_custom_resources": {
        "endpoint": "/core/kubernetes/apiresources/:project_id",
        "method": "GET",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_get_crd": {
        "endpoint": "/core/kubernetes/resource/:project_id",
        "method": "GET",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_create_crd": {
        "endpoint": "/core/kubernetes/resource",
        "method": "POST",
        "required_params": [
            {"name": "crd_data", "type": "object", "desc": "CRD data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_edit_crd": {
        "endpoint": "/core/kubernetes/resource/:project_id/:name",
        "method": "PATCH",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "name", "type": "string", "desc": "CRD name"},
            {"name": "crd_data", "type": "object", "desc": "CRD data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_delete_crd": {
        "endpoint": "/core/kubernetes/resource/:project_id/:name",
        "method": "DELETE",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "name", "type": "string", "desc": "CRD name"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_get_image_os": {
        "endpoint": "/core/cluster-image-os",
        "method": "GET",
        "required_params": [],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_get_vm_flavor_type": {
        "endpoint": "/core/virtual-machine/flavor_type",
        "method": "GET",
        "required_params": [],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_get_vm_gpu": {
        "endpoint": "/core/virtual-machine/gpu/:project_id",
        "method": "GET",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_get_vm_storage_class": {
        "endpoint": "/core/virtual-machine/storage-class/:project_id",
        "method": "GET",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_get_vm_flavor": {
        "endpoint": "/core/virtual-machine/flavor/:flavorType_id",
        "method": "GET",
        "required_params": [
            {"name": "flavorType_id", "type": "string", "desc": "Flavor Type ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_create_vm": {
        "endpoint": "/core/virtual-machine",
        "method": "POST",
        "required_params": [
            {"name": "vm_data", "type": "object", "desc": "VM data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_create_vm_yaml": {
        "endpoint": "/core/virtual-machine/yaml",
        "method": "POST",
        "required_params": [
            {"name": "vm_yaml_data", "type": "object", "desc": "VM YAML data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_get_vm": {
        "endpoint": "/core/virtual-machine/list/all",
        "method": "GET",
        "required_params": [],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_edit_vm_yaml": {
        "endpoint": "/core/virtual-machine/yaml/:project_id/:namespace/:name",
        "method": "PUT",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "namespace", "type": "string", "desc": "Namespace"},
            {"name": "name", "type": "string", "desc": "VM name"},
            {"name": "vm_yaml_data", "type": "object", "desc": "VM YAML data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_delete_vm": {
        "endpoint": "/core/virtual-machine/delete",
        "method": "POST",
        "required_params": [
            {"name": "vm_delete_data", "type": "object", "desc": "VM delete data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_reboot_vm": {
        "endpoint": "/core/virtual-machine/reboot",
        "method": "POST",
        "required_params": [
            {"name": "vm_reboot_data", "type": "object", "desc": "VM reboot data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_turn_off_vm": {
        "endpoint": "/core/virtual-machine/turn-off/vm",
        "method": "POST",
        "required_params": [
            {"name": "vm_turn_off_data", "type": "object", "desc": "VM turn off data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_turn_on_vm": {
        "endpoint": "/core/virtual-machine/turn-on/vm",
        "method": "POST",
        "required_params": [
            {"name": "vm_turn_on_data", "type": "object", "desc": "VM turn on data"},
        ],
        "optional_params": [],
    },
    # --- Final batch of missing tools from endpoint map ---
    "mcp_cldkctl_cldkctl_registry_quota": {
        "endpoint": "/core/dekaregistry/v2/project/quota/:project_id",
        "method": "GET",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_cert": {
        "endpoint": "/core/dekaregistry/v1/registry/downloadcrt",
        "method": "GET",
        "required_params": [],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_create": {
        "endpoint": "/core/dekaregistry/v2/registry",
        "method": "POST",
        "required_params": [
            {"name": "registry_data", "type": "object", "desc": "Registry data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_update": {
        "endpoint": "/core/dekaregistry/v2/registry/:registry_id",
        "method": "PUT",
        "required_params": [
            {"name": "registry_id", "type": "string", "desc": "Registry ID"},
            {"name": "registry_data", "type": "object", "desc": "Registry data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_logs": {
        "endpoint": "/core/dekaregistry/v2/registry/:registry_id/logs",
        "method": "GET",
        "required_params": [
            {"name": "registry_id", "type": "string", "desc": "Registry ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_labels": {
        "endpoint": "/core/dekaregistry/v1/registry/lislabels/:organization_id/:user_id/:project_id/:registry_id",
        "method": "GET",
        "required_params": [
            {"name": "organization_id", "type": "string", "desc": "Organization ID"},
            {"name": "user_id", "type": "string", "desc": "User ID"},
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "registry_id", "type": "string", "desc": "Registry ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_labels_update": {
        "endpoint": "/core/dekaregistry/v1/registry/updatelabels/:organization_id/:user_id/:project_id/:registry_id",
        "method": "PUT",
        "required_params": [
            {"name": "organization_id", "type": "string", "desc": "Organization ID"},
            {"name": "user_id", "type": "string", "desc": "User ID"},
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "registry_id", "type": "string", "desc": "Registry ID"},
            {"name": "labels_data", "type": "object", "desc": "Labels data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_labels_create": {
        "endpoint": "/core/dekaregistry/v1/registry/createlabels/:organization_id/:user_id/:project_id/:registry_id",
        "method": "POST",
        "required_params": [
            {"name": "organization_id", "type": "string", "desc": "Organization ID"},
            {"name": "user_id", "type": "string", "desc": "User ID"},
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "registry_id", "type": "string", "desc": "Registry ID"},
            {"name": "labels_data", "type": "object", "desc": "Labels data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_labels_delete": {
        "endpoint": "/core/dekaregistry/v1/registry/deletelabels/:organization_id/:user_id/:project_id/:labels_id/:registry_id",
        "method": "DELETE",
        "required_params": [
            {"name": "organization_id", "type": "string", "desc": "Organization ID"},
            {"name": "user_id", "type": "string", "desc": "User ID"},
            {"name": "project_id", "type": "string", "desc": "Project ID"},
            {"name": "labels_id", "type": "string", "desc": "Labels ID"},
            {"name": "registry_id", "type": "string", "desc": "Registry ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_tag_list": {
        "endpoint": "/core/dekaregistry/v2/tag/:registry_id",
        "method": "GET",
        "required_params": [
            {"name": "registry_id", "type": "string", "desc": "Registry ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_tag_create": {
        "endpoint": "/core/dekaregistry/v2/tag/:registry_id",
        "method": "POST",
        "required_params": [
            {"name": "registry_id", "type": "string", "desc": "Registry ID"},
            {"name": "tag_data", "type": "object", "desc": "Tag data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_tag_update": {
        "endpoint": "/core/dekaregistry/v2/tag/detail/:tag_id",
        "method": "PUT",
        "required_params": [
            {"name": "tag_id", "type": "string", "desc": "Tag ID"},
            {"name": "tag_data", "type": "object", "desc": "Tag data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_tag_delete": {
        "endpoint": "/core/dekaregistry/v2/tag/detail/:tag_id",
        "method": "DELETE",
        "required_params": [
            {"name": "tag_id", "type": "string", "desc": "Tag ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_tag_disable": {
        "endpoint": "/core/dekaregistry/v2/tag/detail/:tag_id/disable",
        "method": "POST",
        "required_params": [
            {"name": "tag_id", "type": "string", "desc": "Tag ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_tag_enable": {
        "endpoint": "/core/dekaregistry/v2/tag/detail/:tag_id/enable",
        "method": "POST",
        "required_params": [
            {"name": "tag_id", "type": "string", "desc": "Tag ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_available_member": {
        "endpoint": "/core/dekaregistry/v2/project/member/:project_id",
        "method": "GET",
        "required_params": [
            {"name": "project_id", "type": "string", "desc": "Project ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_show_password": {
        "endpoint": "/core/dekaregistry/v2/user/password/show",
        "method": "POST",
        "required_params": [
            {"name": "password_data", "type": "object", "desc": "Password data"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_repository_list": {
        "endpoint": "/core/dekaregistry/v2/repository",
        "method": "GET",
        "required_params": [],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_artifact_detail": {
        "endpoint": "/core/dekaregistry/v2/artifact/:artifact_id",
        "method": "GET",
        "required_params": [
            {"name": "artifact_id", "type": "string", "desc": "Artifact ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_artifact_add_label": {
        "endpoint": "/core/dekaregistry/v2/artifact/:artifact_id/assign-label/:label_id",
        "method": "PATCH",
        "required_params": [
            {"name": "artifact_id", "type": "string", "desc": "Artifact ID"},
            {"name": "label_id", "type": "string", "desc": "Label ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_artifact_remove_label": {
        "endpoint": "/core/dekaregistry/v2/artifact/:artifact_id/unassign-label/:label_id",
        "method": "PATCH",
        "required_params": [
            {"name": "artifact_id", "type": "string", "desc": "Artifact ID"},
            {"name": "label_id", "type": "string", "desc": "Label ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_artifact_scan": {
        "endpoint": "/core/dekaregistry/v2/artifact/:artifact_id/scan",
        "method": "POST",
        "required_params": [
            {"name": "artifact_id", "type": "string", "desc": "Artifact ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_artifact_stop_scan": {
        "endpoint": "/core/dekaregistry/v2/artifact/:artifact_id/stop-scan",
        "method": "POST",
        "required_params": [
            {"name": "artifact_id", "type": "string", "desc": "Artifact ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_artifact_tags": {
        "endpoint": "/core/dekaregistry/v2/artifact/:artifact_id/tag",
        "method": "GET",
        "required_params": [
            {"name": "artifact_id", "type": "string", "desc": "Artifact ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_artifact_delete_tag": {
        "endpoint": "/core/dekaregistry/v2/artifact/:artifact_id/tag/:tag",
        "method": "DELETE",
        "required_params": [
            {"name": "artifact_id", "type": "string", "desc": "Artifact ID"},
            {"name": "tag", "type": "string", "desc": "Tag name"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_registry_artifact_add_tag": {
        "endpoint": "/core/dekaregistry/v2/artifact/:artifact_id/tag/:tag",
        "method": "POST",
        "required_params": [
            {"name": "artifact_id", "type": "string", "desc": "Artifact ID"},
            {"name": "tag", "type": "string", "desc": "Tag name"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_notebook_images": {
        "endpoint": "/core/deka-notebook/images",
        "method": "GET",
        "required_params": [],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_superadmin_project_list": {
        "endpoint": "/core/superadmin/list/manageorgproject",
        "method": "GET",
        "required_params": [],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_superadmin_org_detail": {
        "endpoint": "/core/superadmin/manageorg/:organization_id",
        "method": "GET",
        "required_params": [
            {"name": "organization_id", "type": "string", "desc": "Organization ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_superadmin_balance_detail": {
        "endpoint": "/core/superadmin/balance/accumulated/:organization_id/:project_id",
        "method": "GET",
        "required_params": [
            {"name": "organization_id", "type": "string", "desc": "Organization ID"},
            {"name": "project_id", "type": "string", "desc": "Project ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_superadmin_billing_invoice_sme": {
        "endpoint": "/core/superadmin/balance/history/invoice/:organization_id",
        "method": "GET",
        "required_params": [
            {"name": "organization_id", "type": "string", "desc": "Organization ID"},
        ],
        "optional_params": [],
    },
    "mcp_cldkctl_cldkctl_superadmin_billing_invoice_enterprise": {
        "endpoint": "/core/superadmin/invoice/:organization_id/:project_id",
        "method": "GET",
        "required_params": [
            {"name": "organization_id", "type": "string", "desc": "Organization ID"},
            {"name": "project_id", "type": "string", "desc": "Project ID"},
        ],
        "optional_params": [],
    },
} 


#!/usr/bin/env python3
"""
Test the /api/cldkctl/auth endpoint.
"""

import requests
import json

def test_api_endpoint():
    """Test the /api/cldkctl/auth endpoint."""
    print("Testing /api/cldkctl/auth endpoint...")
    
    # Test with the real login token
    sample_token = "cldkctl_Np5ShM0TQKFTHRxGqFISxlrzcv7UqpDA"
    production_url = "https://ai.cloudeka.id"
    
    print(f"Login token: {sample_token}")
    print(f"Production URL: {production_url}")
    
    # Test the api auth endpoint
    url = production_url + "/api/cldkctl/auth"
    payload = {"token": sample_token}
    
    try:
        resp = requests.post(url, json=payload, headers={'Content-Type': 'application/json'})
        print(f"Response status: {resp.status_code}")
        print(f"Response: {resp.text}")
        
        if resp.status_code == 200:
            data = resp.json()
            jwt_token = data.get("data", {}).get("token")
            if jwt_token:
                print(f"✅ Success! JWT token obtained: {jwt_token[:50]}...")
                return True
            else:
                print("❌ No JWT token in response")
                return False
        else:
            print("❌ Failed")
            return False
    except Exception as e:
        print(f"❌ Error: {e}")
        return False

if __name__ == "__main__":
    test_api_endpoint() 


#!/usr/bin/env python3
"""
Test script for cldkctl MCP authentication flow.
This script tests the token exchange logic without running the full MCP server.
"""

import sys
import os
import requests
import base64
import json

# Add the mcp_cldkctl directory to the path
sys.path.insert(0, os.path.join(os.path.dirname(__file__), 'mcp_cldkctl'))

from server import exchange_login_token_for_jwt, TokenCache

def debug_auth_flow():
    """Debug the authentication flow with detailed request/response logging."""
    print("Debugging cldkctl authentication flow...")
    
    # Test with the real login token
    sample_token = "cldkctl_Np5ShM0TQKFTHRxGqFISxlrzcv7UqpDA"
    base_url = "https://ai.cloudeka.id"
    
    print(f"Login token: {sample_token}")
    print(f"Base URL: {base_url}")
    
    # Prepare the request
    url = base_url + "/core/cldkctl/auth"
    payload = {"token": sample_token}
    
    print(f"Request URL: {url}")
    print(f"Request payload: {json.dumps(payload, indent=2)}")
    print(f"Request headers: {{'Content-Type': 'application/json'}}")
    
    try:
        # Make the request
        resp = requests.post(url, json=payload, headers={'Content-Type': 'application/json'})
        
        print(f"Response status: {resp.status_code}")
        print(f"Response headers: {dict(resp.headers)}")
        
        if resp.status_code != 200:
            print(f"Error response body: {resp.text}")
            
            # Try to parse as JSON
            try:
                error_data = resp.json()
                print(f"Error response JSON: {json.dumps(error_data, indent=2)}")
            except:
                print("Error response is not JSON")
        else:
            print(f"Success response body: {resp.text}")
            
            # Try to parse as JSON
            try:
                success_data = resp.json()
                print(f"Success response JSON: {json.dumps(success_data, indent=2)}")
            except:
                print("Success response is not JSON")
                
    except Exception as e:
        print(f"Request failed with exception: {e}")

if __name__ == "__main__":
    debug_auth_flow() 


#!/usr/bin/env python3
"""
Test script for mcp-cldkctl package
"""

import subprocess
import sys
import os

def test_import():
    """Test that the package can be imported"""
    try:
        import mcp_cldkctl
        print("✅ Package import successful")
        return True
    except ImportError as e:
        print(f"❌ Package import failed: {e}")
        return False

def test_server_import():
    """Test that the server can be imported"""
    try:
        from mcp_cldkctl.server import CldkctlMCPServer
        print("✅ Server import successful")
        return True
    except ImportError as e:
        print(f"❌ Server import failed: {e}")
        return False

def test_server_initialization():
    """Test that the server can be initialized"""
    try:
        from mcp_cldkctl.server import CldkctlMCPServer
        server = CldkctlMCPServer()
        print("✅ Server initialization successful")
        return True
    except Exception as e:
        print(f"❌ Server initialization failed: {e}")
        return False

def test_uvx_command():
    """Test that the package can be run with uvx"""
    try:
        # Test if uvx is available
        result = subprocess.run(
            ["uvx", "--help"], 
            capture_output=True, 
            text=True, 
            timeout=10
        )
        if result.returncode != 0:
            print("❌ uvx not available")
            return False
        
        # Test if our package is available
        result = subprocess.run(
            ["uvx", "mcp-cldkctl", "--help"], 
            capture_output=True, 
            text=True, 
            timeout=10
        )
        if result.returncode == 0:
            print("✅ uvx command successful")
            return True
        else:
            print(f"❌ uvx command failed: {result.stderr}")
            return False
    except subprocess.TimeoutExpired:
        print("❌ uvx command timed out")
        return False
    except FileNotFoundError:
        print("❌ uvx not found in PATH")
        return False
    except Exception as e:
        print(f"❌ uvx command error: {e}")
        return False

def test_environment_variables():
    """Test environment variable handling"""
    try:
        from mcp_cldkctl.server import CldkctlMCPServer
        
        # Test with no environment variables
        server = CldkctlMCPServer()
        
        # Test with environment variables
        os.environ['CLDKCTL_TOKEN'] = 'test_token'
        os.environ['CLDKCTL_BASE_URL'] = 'https://test.cloudeka.id'
        
        server2 = CldkctlMCPServer()
        
        print("✅ Environment variable handling successful")
        return True
    except Exception as e:
        print(f"❌ Environment variable handling failed: {e}")
        return False

def main():
    """Run all tests"""
    print("🧪 Testing mcp-cldkctl package...")
    print("=" * 50)
    
    tests = [
        ("Package Import", test_import),
        ("Server Import", test_server_import),
        ("Server Initialization", test_server_initialization),
        ("Environment Variables", test_environment_variables),
        ("uvx Command", test_uvx_command),
    ]
    
    passed = 0
    total = len(tests)
    
    for test_name, test_func in tests:
        print(f"\n🔍 Running {test_name}...")
        if test_func():
            passed += 1
        else:
            print(f"   Failed: {test_name}")
    
    print("\n" + "=" * 50)
    print(f"📊 Test Results: {passed}/{total} tests passed")
    
    if passed == total:
        print("🎉 All tests passed! Package is ready for deployment.")
        return 0
    else:
        print("❌ Some tests failed. Please fix issues before deployment.")
        return 1

if __name__ == "__main__":
    sys.exit(main()) 


#!/usr/bin/env python3
"""
Test script to check if production URL works with fresh token.
"""

import requests
import json

def test_production_with_fresh_token():
    """Test if production works with a fresh token."""
    print("Testing production URL with fresh token...")
    
    # Test with the real login token
    sample_token = "cldkctl_Np5ShM0TQKFTHRxGqFISxlrzcv7UqpDA"
    production_url = "https://ai.cloudeka.id"
    
    print(f"Login token: {sample_token}")
    print(f"Production URL: {production_url}")
    
    # Test the auth endpoint
    url = production_url + "/core/cldkctl/auth"
    payload = {"token": sample_token}
    
    try:
        resp = requests.post(url, json=payload, headers={'Content-Type': 'application/json'})
        print(f"Response status: {resp.status_code}")
        print(f"Response: {resp.text}")
        
        if resp.status_code == 200:
            print("✅ Production URL works!")
            return True
        else:
            print("❌ Production URL still has issues")
            return False
    except Exception as e:
        print(f"❌ Error: {e}")
        return False

def test_go_cli_behavior():
    """Test to understand how Go CLI might be different."""
    print("\n=== Testing Go CLI behavior simulation ===")
    
    # The Go CLI might be using a different approach
    # Let's test if it's using a different endpoint or method
    
    sample_token = "cldkctl_Np5ShM0TQKFTHRxGqFISxlrzcv7UqpDA"
    production_url = "https://ai.cloudeka.id"
    
    # Test 1: Check if there's a different auth endpoint
    endpoints = [
        "/core/cldkctl/auth",
        "/core/user/auth", 
        "/core/auth/cldkctl",
        "/api/cldkctl/auth"
    ]
    
    for endpoint in endpoints:
        url = production_url + endpoint
        payload = {"token": sample_token}
        
        try:
            resp = requests.post(url, json=payload, headers={'Content-Type': 'application/json'})
            print(f"Endpoint {endpoint}: {resp.status_code}")
            if resp.status_code == 200:
                print(f"✅ Found working endpoint: {endpoint}")
                return True
        except Exception as e:
            print(f"Endpoint {endpoint}: Error - {e}")
    
    return False

if __name__ == "__main__":
    test_production_with_fresh_token()
    test_go_cli_behavior() 


"""MCP server for Cloudeka cldkctl CLI integration (Testing Version)"""

__version__ = "0.1.36"

import os
import json
import pathlib

# Try to load token and base_url from mcp.json at import time
CLDKCTL_TOKEN = None
CLDKCTL_BASE_URL = None

def _load_token_from_mcp_json():
    config_path = os.path.expanduser("~/.cursor/mcp.json")
    if not os.path.exists(config_path):
        config_path = str(pathlib.Path.home() / ".cursor" / "mcp.json")
    try:
        with open(config_path, "r") as f:
            data = json.load(f)
            token = data.get("mcpServers", {}).get("cldkctl", {}).get("env", {}).get("CLDKCTL_TOKEN")
            base_url = data.get("mcpServers", {}).get("cldkctl", {}).get("env", {}).get("CLDKCTL_BASE_URL")
            return token, base_url
    except Exception:
        return None, None

CLDKCTL_TOKEN, CLDKCTL_BASE_URL = _load_token_from_mcp_json()

# mcp_cldkctl package 


#!/usr/bin/env python3
"""
MCP Server for Cloudeka CLI (cldkctl) functionality.
"""

import asyncio
import base64
import json
import os
import sys
from datetime import datetime, timedelta
from typing import Any, Dict, Optional
from importlib import metadata

import requests
from mcp.server import Server
from mcp.server.models import InitializationOptions
from mcp.server.stdio import stdio_server
from mcp.types import (
    CallToolResult,
    Tool,
    TextContent,
    Content,
)
from . import CLDKCTL_TOKEN as PKG_CLDKCTL_TOKEN, CLDKCTL_BASE_URL as PKG_CLDKCTL_BASE_URL

# Import the tool parameter map
try:
    from .tool_param_map import TOOL_PARAM_MAP
except ImportError:
    # Fallback if the file doesn't exist
    TOOL_PARAM_MAP = {}


class SimpleNotificationOptions:
    """Simple notification options class for MCP server capabilities."""

    def __init__(self, prompts_changed=False, resources_changed=False, tools_changed=False):
        self.prompts_changed = prompts_changed
        self.resources_changed = resources_changed
        self.tools_changed = tools_changed


# Initialize the server
server = Server("cldkctl")

# Configuration
PRODUCTION_URL = "https://ai.cloudeka.id"
STAGING_URL = "https://staging.ai.cloudeka.id"
CACHE_FILE = os.path.expanduser("~/.cldkctl/mcp_cache.json")
CACHE_DIR = os.path.expanduser("~/.cldkctl")

# Ensure cache directory exists
os.makedirs(CACHE_DIR, exist_ok=True)

# Global state for authentication and environment
auth_cache = {
    "jwt_token": None,
    "login_payload": None,
    "expires_at": None,
    "user_info": None,
    "environment": "production",
    "base_url": PRODUCTION_URL,
}

# Environment configuration
current_base_url = PRODUCTION_URL  # Default to production
environment_name = "production"


def load_cache():
    """Load cached authentication data."""
    global auth_cache, current_base_url, environment_name
    try:
        if os.path.exists(CACHE_FILE):
            with open(CACHE_FILE, "r") as f:
                cached_data = json.load(f)
                # Check if token is still valid
                if cached_data.get("expires_at"):
                    expires_at = datetime.fromisoformat(cached_data["expires_at"])
                    if datetime.now() < expires_at:
                        auth_cache.update(cached_data)
                        current_base_url = auth_cache["base_url"]
                        environment_name = auth_cache["environment"]
                        print(f"Loaded cached auth data, expires at {expires_at}", file=sys.stderr)
                        return True
                    else:
                        print("Cached token expired", file=sys.stderr)
                return False
    except Exception as e:
        print(f"Error loading cache: {e}", file=sys.stderr)
    return False


def save_cache():
    """Save authentication data to cache."""
    try:
        with open(CACHE_FILE, "w") as f:
            json.dump(auth_cache, f, default=str)
    except Exception as e:
        print(f"Error saving cache: {e}", file=sys.stderr)


def authenticate_with_token(token: str, force_staging: bool = False) -> bool:
    """Authenticate using a cldkctl token and get JWT."""
    global auth_cache, current_base_url, environment_name

    # Determine which URL to use
    if force_staging:
        base_url = STAGING_URL
        env_name = "staging"
    else:
        base_url = PRODUCTION_URL
        env_name = "production"

    print(f"Authenticating with {env_name} environment: {base_url}", file=sys.stderr)
    url = f"{base_url}/core/cldkctl/auth"
    payload = {"token": token}

    try:
        response = requests.post(url, json=payload, headers={"Content-Type": "application/json"})
        print(f"Auth response status: {response.status_code}", file=sys.stderr)

        if response.status_code == 200:
            data = response.json()
            jwt_token = data.get("data", {}).get("token")
            if jwt_token:
                # Update global state
                current_base_url = base_url
                environment_name = env_name
                # Cache the authentication data
                auth_cache["jwt_token"] = jwt_token
                auth_cache["login_payload"] = base64.b64encode(json.dumps(payload).encode()).decode()
                auth_cache["expires_at"] = (datetime.now() + timedelta(hours=24)).isoformat()
                auth_cache["user_info"] = data.get("data", {})
                auth_cache["environment"] = env_name
                auth_cache["base_url"] = base_url
                save_cache()
                print(f"Authentication successful with {env_name}", file=sys.stderr)
                return True
            else:
                print("No JWT token in response", file=sys.stderr)
                return False
        elif response.status_code == 400 and "pq: relation \"cldkctl_tokens\" does not exist" in response.text:
            print("Production backend has database issue, trying staging...", file=sys.stderr)
            if not force_staging:
                return authenticate_with_token(token, force_staging=True)
            else:
                print("Staging also failed with the same issue.", file=sys.stderr)
                return False
        else:
            print(f"Authentication failed: {response.status_code} - {response.text}", file=sys.stderr)
            return False
    except requests.RequestException as e:
        print(f"Authentication request error: {e}", file=sys.stderr)
        if not force_staging:
            print("Trying staging as fallback...", file=sys.stderr)
            return authenticate_with_token(token, force_staging=True)
        return False


def get_auth_headers() -> Dict[str, str]:
    """Get headers with authentication token."""
    if not auth_cache.get("jwt_token"):
        raise Exception("Not authenticated. Please authenticate first.")

    # Check for token expiration
    expires_at_str = auth_cache.get("expires_at")
    if expires_at_str:
        if datetime.now() >= datetime.fromisoformat(expires_at_str):
            print("Token expired, attempting re-authentication", file=sys.stderr)
            if auth_cache.get("login_payload"):
                login_data = json.loads(base64.b64decode(auth_cache["login_payload"]).decode())
                if not authenticate_with_token(login_data["token"], force_staging=(environment_name == "staging")):
                    raise Exception("Re-authentication failed")
            else:
                raise Exception("Token expired and no login payload to re-authenticate.")

    return {
        "Authorization": f"Bearer {auth_cache['jwt_token']}",
        "Content-Type": "application/json",
    }


def make_authenticated_request(method: str, endpoint: str, data: Optional[Dict] = None, params: Optional[Dict] = None) -> Dict:
    """Make an authenticated request to the API."""
    url = f"{current_base_url}{endpoint}"
    try:
        headers = get_auth_headers()
        # For GET requests, do not send a body (data)
        if method.upper() == "GET":
            response = requests.request(method, url, headers=headers, params=params)
        else:
            response = requests.request(method, url, headers=headers, json=data, params=params)
        response.raise_for_status()
        return response.json()
    except Exception as e:
        print(f"Authenticated request failed: {e}", file=sys.stderr)
        # In case of an error, return a structured error message
        return {"error": True, "message": str(e), "status_code": getattr(e.response, "status_code", None)}


def get_tool_definitions() -> list[Tool]:
    """Generate tool definitions from TOOL_PARAM_MAP."""
    tools = []
    
    # Add basic authentication tools that are handled specially
    tools.extend([
        Tool(
            name="auth",
            description="Authenticate with a cldkctl token to get JWT access",
            inputSchema={
                "type": "object",
                "properties": {
                    "token": {
                        "type": "string",
                        "description": "Your cldkctl token (starts with 'cldkctl_')"
                    },
                    "force_staging": {
                        "type": "boolean",
                        "description": "Force using staging environment (default: false, will auto-fallback if production fails)"
                    }
                },
                "required": ["token"]
            }
        ),
    ])
    
    # Generate tools from TOOL_PARAM_MAP
    for tool_name, tool_info in TOOL_PARAM_MAP.items():
        # Remove the mcp_cldkctl_ prefix for the tool name
        clean_name = tool_name.replace("mcp_cldkctl_", "")
        
        # Build properties and required fields from the tool info
        properties = {}
        required = []
        
        # Add required parameters
        for param in tool_info.get("required_params", []):
            param_name = param["name"]
            param_type = param["type"]
            param_desc = param.get("desc", "")
            
            # Map parameter types to JSON schema types
            if param_type == "string":
                schema_type = "string"
            elif param_type == "integer":
                schema_type = "integer"
            elif param_type == "boolean":
                schema_type = "boolean"
            elif param_type == "list":
                schema_type = "array"
            elif param_type == "dict" or param_type == "object":
                schema_type = "object"
            else:
                schema_type = "string"  # Default to string
            
            properties[param_name] = {
                "type": schema_type,
                "description": param_desc
            }
            required.append(param_name)
        
        # Add optional parameters
        for param in tool_info.get("optional_params", []):
            param_name = param["name"]
            param_type = param["type"]
            param_desc = param.get("desc", "")
            
            # Map parameter types to JSON schema types
            if param_type == "string":
                schema_type = "string"
            elif param_type == "integer":
                schema_type = "integer"
            elif param_type == "boolean":
                schema_type = "boolean"
            elif param_type == "list":
                schema_type = "array"
            elif param_type == "dict" or param_type == "object":
                schema_type = "object"
            else:
                schema_type = "string"  # Default to string
            
            properties[param_name] = {
                "type": schema_type,
                "description": param_desc
            }
        
        # Create the tool
        tool = Tool(
            name=clean_name,
            description=f"Call the {clean_name} endpoint",
            inputSchema={
                "type": "object",
                "properties": properties,
                "required": required
            }
        )
        tools.append(tool)
    
    return tools


@server.list_tools()
async def handle_list_tools() -> list[Tool]:
    """List all available tools."""
    try:
        tools = get_tool_definitions()
        print(f"Successfully loaded {len(tools)} tools.", file=sys.stderr)
        return tools
    except Exception as e:
        print(f"!!!!!!!! ERROR GETTING TOOL DEFINITIONS !!!!!!!!", file=sys.stderr)
        print(f"Exception: {e}", file=sys.stderr)
        import traceback
        traceback.print_exc(file=sys.stderr)
        return []


def format_response(data: Any) -> str:
    """Formats API response data for display."""
    if isinstance(data, dict) and data.get("error"):
        return f"❌ API Error: {data.get('message', 'Unknown error')}"
    return f"```json\n{json.dumps(data, indent=2)}\n```"


@server.call_tool()
async def handle_call_tool(name: str, arguments: Dict[str, Any]) -> list[Content]:
    """Handle tool calls."""
    global current_base_url, environment_name
    
    # Normalize tool name by removing mcp_cldkctl_ prefix if present
    tool_name = name.replace("mcp_cldkctl_", "") if name.startswith("mcp_cldkctl_") else name
    
    try:
        # Handle authentication tools
        if tool_name == "auth":
            token = arguments["token"]
            force_staging = arguments.get("force_staging", False)
            if authenticate_with_token(token, force_staging):
                env_info = f" ({environment_name})" if environment_name != "production" else ""
                user_info = auth_cache.get('user_info', {})
                text = (
                    f"✅ Authentication successful{env_info}!\n\n"
                    f"User: {user_info.get('name', 'Unknown')}\n"
                    f"Role: {user_info.get('role', 'Unknown')}\n"
                    f"Organization: {user_info.get('organization_id', 'None')}\n"
                    f"Environment: {environment_name}\n"
                    f"Base URL: {current_base_url}"
                )
                return [TextContent(type="text", text=text)]
            else:
                return [TextContent(type="text", text="❌ Authentication failed.")]

        # Handle all other tools from TOOL_PARAM_MAP
        full_tool_name = f"mcp_cldkctl_{tool_name}"
        if full_tool_name in TOOL_PARAM_MAP:
            tool_info = TOOL_PARAM_MAP[full_tool_name]
            endpoint = tool_info["endpoint"]
            method = tool_info["method"]

            # Special case: registry list should always send page=1 and project-id if provided
            if full_tool_name == "mcp_cldkctl_cldkctl_registry_list":
                params = {"page": "1"}
                if "project-id" in arguments:
                    params["project-id"] = arguments["project-id"]
                data = make_authenticated_request(method, endpoint, params=params)
                return [TextContent(type="text", text=format_response(data))]
            
            # Handle path parameters in the endpoint
            try:
                # Replace path parameters like :param_name with actual values
                for param_name, param_value in arguments.items():
                    if f":{param_name}" in endpoint:
                        endpoint = endpoint.replace(f":{param_name}", str(param_value))
                
                # Determine what to send as body vs query params
                if method.upper() in ["POST", "PUT", "PATCH"]:
                    # For POST/PUT/PATCH, send arguments as JSON body
                    data = make_authenticated_request(method, endpoint, data=arguments)
                else:
                    # For GET/DELETE, send arguments as query parameters
                    data = make_authenticated_request(method, endpoint, params=arguments)
                
                return [TextContent(type="text", text=format_response(data))]
                
            except KeyError as e:
                return [TextContent(type="text", text=f"❌ Missing required parameter: {e}")]
            except Exception as e:
                return [TextContent(type="text", text=f"❌ Error executing {tool_name}: {str(e)}")]
        else:
            # Try without the prefix as well
            if tool_name in TOOL_PARAM_MAP:
                tool_info = TOOL_PARAM_MAP[tool_name]
                endpoint = tool_info["endpoint"]
                method = tool_info["method"]
                
                try:
                    # Replace path parameters like :param_name with actual values
                    for param_name, param_value in arguments.items():
                        if f":{param_name}" in endpoint:
                            endpoint = endpoint.replace(f":{param_name}", str(param_value))
                    
                    # Determine what to send as body vs query params
                    if method.upper() in ["POST", "PUT", "PATCH"]:
                        # For POST/PUT/PATCH, send arguments as JSON body
                        data = make_authenticated_request(method, endpoint, data=arguments)
                    else:
                        # For GET/DELETE, send arguments as query parameters
                        data = make_authenticated_request(method, endpoint, params=arguments)
                    
                    return [TextContent(type="text", text=format_response(data))]
                    
                except KeyError as e:
                    return [TextContent(type="text", text=f"❌ Missing required parameter: {e}")]
                except Exception as e:
                    return [TextContent(type="text", text=f"❌ Error executing {tool_name}: {str(e)}")]
            else:
                available_tools = [name.replace("mcp_cldkctl_", "") for name in TOOL_PARAM_MAP.keys()]
                return [TextContent(type="text", text=f"❌ Unknown tool: {tool_name}. Available tools: {', '.join(available_tools[:20])}...")]

    except Exception as e:
        print(f"Error calling tool {tool_name}: {e}", file=sys.stderr)
        import traceback
        traceback.print_exc(file=sys.stderr)
        return [TextContent(type="text", text=f"❌ Error executing tool '{tool_name}': {str(e)}")]


async def main():
    """Main function."""
    print("Initializing server...", file=sys.stderr)
    try:
        __version__ = metadata.version("mcp-cldkctl")
    except metadata.PackageNotFoundError:
        __version__ = "0.0.0-dev"

    # Token loading logic moved here
    def load_token_from_mcp_json():
        import pathlib
        config_path = os.path.expanduser("~/.cursor/mcp.json")
        if not os.path.exists(config_path):
            config_path = str(pathlib.Path.home() / ".cursor" / "mcp.json")
        try:
            with open(config_path, "r") as f:
                data = json.load(f)
                token = data.get("mcpServers", {}).get("cldkctl", {}).get("env", {}).get("CLDKCTL_TOKEN")
                base_url = data.get("mcpServers", {}).get("cldkctl", {}).get("env", {}).get("CLDKCTL_BASE_URL")
                return token, base_url
        except Exception:
            return None, None

    if not load_cache():
        token = os.environ.get("CLDKCTL_TOKEN")
        base_url = os.environ.get("CLDKCTL_BASE_URL")
        # Use package-level values as fallback if env not set
        if not token:
            token = PKG_CLDKCTL_TOKEN
        if not base_url:
            base_url = PKG_CLDKCTL_BASE_URL
        # If still not set, try reading mcp.json directly (legacy fallback)
        if not token:
            token, base_url = load_token_from_mcp_json()
        if token:
            print("No valid cache found, authenticating with token from environment, package, or mcp.json...", file=sys.stderr)
            force_staging = base_url == STAGING_URL if base_url else False
            authenticate_with_token(token, force_staging=force_staging)
        else:
            print("No valid cache or environment/config token found. You will be prompted for a token on first use.", file=sys.stderr)

    async with stdio_server() as (read_stream, write_stream):
        await server.run(
            read_stream,
            write_stream,
            InitializationOptions(
                server_name="cldkctl",
                server_version=__version__,
                capabilities=server.get_capabilities(
                    notification_options=SimpleNotificationOptions(
                        prompts_changed=False,
                        resources_changed=False,
                        tools_changed=False,
                    ),
                    experimental_capabilities={},
                ),
            ),
        )

if __name__ == "__main__":
    try:
        asyncio.run(main())
    except KeyboardInterrupt:
        print("Server shutting down.", file=sys.stderr)


#!/usr/bin/env python3
"""
Detailed comparison between production and staging authentication.
"""

import requests
import json
import time

def test_endpoint(url, token, description):
    """Test an endpoint and return detailed results."""
    print(f"\n=== Testing {description} ===")
    print(f"URL: {url}")
    print(f"Token: {token[:20]}...")
    
    payload = {"token": token}
    headers = {'Content-Type': 'application/json'}
    
    try:
        start_time = time.time()
        response = requests.post(url, json=payload, headers=headers)
        end_time = time.time()
        
        print(f"Response Time: {end_time - start_time:.2f}s")
        print(f"Status Code: {response.status_code}")
        print(f"Headers: {dict(response.headers)}")
        print(f"Response Length: {len(response.text)} characters")
        
        if response.status_code == 200:
            try:
                data = response.json()
                jwt_token = data.get("data", {}).get("token")
                if jwt_token:
                    print(f"✅ SUCCESS! JWT Token: {jwt_token[:50]}...")
                    return True
                else:
                    print("❌ No JWT token in response")
                    print(f"Response: {json.dumps(data, indent=2)}")
                    return False
            except json.JSONDecodeError:
                print("❌ Invalid JSON response")
                print(f"Response: {response.text[:500]}...")
                return False
        else:
            print(f"❌ Failed with status {response.status_code}")
            print(f"Response: {response.text}")
            return False
            
    except Exception as e:
        print(f"❌ Error: {e}")
        return False

def main():
    """Main test function."""
    token = "cldkctl_Np5ShM0TQKFTHRxGqFISxlrzcv7UqpDA"
    
    # Test production
    production_url = "https://ai.cloudeka.id/core/cldkctl/auth"
    production_success = test_endpoint(production_url, token, "PRODUCTION")
    
    # Test staging
    staging_url = "https://staging.ai.cloudeka.id/core/cldkctl/auth"
    staging_success = test_endpoint(staging_url, token, "STAGING")
    
    print(f"\n=== SUMMARY ===")
    print(f"Production: {'✅ SUCCESS' if production_success else '❌ FAILED'}")
    print(f"Staging: {'✅ SUCCESS' if staging_success else '❌ FAILED'}")
    
    if production_success and staging_success:
        print("Both environments work! The issue might be intermittent.")
    elif staging_success and not production_success:
        print("Staging works but production doesn't. This suggests a backend issue.")
    elif production_success and not staging_success:
        print("Production works but staging doesn't. This is unexpected.")
    else:
        print("Neither environment works. The token might be invalid or expired.")

if __name__ == "__main__":
    main() 